@@ -41,6 +41,9 @@ along with BTFS.  If not, see <http://www.gnu.org/licenses/>.
 #include <libtorrent/alert_types.hpp>
 #include <libtorrent/magnet_uri.hpp>
 #include <libtorrent/version.hpp>
+#if LIBTORRENT_VERSION_NUM >= 10200
+#include <libtorrent/torrent_flags.hpp>
+#endif
 #pragma GCC diagnostic pop
 
 #include <curl/curl.h>
@@ -86,20 +89,16 @@ move_to_next_unfinished(int& piece, int num_pieces) {
 
 static void
 jump(int piece, int size) {
-#if LIBTORRENT_VERSION_NUM < 10000
-	libtorrent::torrent_info ti = handle.get_torrent_info();
-#else
-	libtorrent::torrent_info ti = *handle.torrent_file();
-#endif
+	auto ti = handle.torrent_file();
 
 	int tail = piece;
 
-	if (!move_to_next_unfinished(tail, ti.num_pieces()))
+	if (!move_to_next_unfinished(tail, ti->num_pieces()))
 		return;
 
 	cursor = tail;
 
-	int pl = ti.piece_length();
+	int pl = ti->piece_length();
 
 	for (int b = 0; b < 16 * pl; b += pl) {
 		handle.piece_priority(tail++, 7);
@@ -116,24 +115,20 @@ advance() {
 }
 
 Read::Read(char *buf, int index, off_t offset, size_t size) {
-#if LIBTORRENT_VERSION_NUM < 10000
-	libtorrent::torrent_info ti = handle.get_torrent_info();
-#else
-	libtorrent::torrent_info ti = *handle.torrent_file();
-#endif
+	auto ti = handle.torrent_file();
 
 #if LIBTORRENT_VERSION_NUM < 10100
-	int64_t file_size = ti.file_at(index).size;
+	int64_t file_size = ti->file_at(index).size;
 #else
-	int64_t file_size = ti.files().file_size(index);
+	int64_t file_size = ti->files().file_size(index);
 #endif
 
 	while (size > 0 && offset < file_size) {
-		libtorrent::peer_request part = ti.map_file(index, offset,
+		libtorrent::peer_request part = ti->map_file(index, offset,
 			(int) size);
 
 		part.length = std::min(
-			ti.piece_size(part.piece) - part.start,
+			ti->piece_size(part.piece) - part.start,
 			part.length);
 
 		parts.push_back(Part(part, buf));
@@ -199,25 +194,21 @@ static void
 setup() {
 	printf("Got metadata. Now ready to start downloading.\n");
 
-#if LIBTORRENT_VERSION_NUM < 10000
-	libtorrent::torrent_info ti = handle.get_torrent_info();
-#else
-	libtorrent::torrent_info ti = *handle.torrent_file();
-#endif
+	auto ti = handle.torrent_file();
 
 	if (params.browse_only)
 		handle.pause();
 
-	for (int i = 0; i < ti.num_files(); ++i) {
+	for (int i = 0; i < ti->num_files(); ++i) {
 		// Initially, don't download anything
 		handle.file_priority(i, 0);
 
 		std::string parent("");
 
 #if LIBTORRENT_VERSION_NUM < 10100
-		char *p = strdup(ti.file_at(i).path.c_str());
+		char *p = strdup(ti->file_at(i).path.c_str());
 #else
-		char *p = strdup(ti.files().file_path(i).c_str());
+		char *p = strdup(ti->files().file_path(i).c_str());
 #endif
 
 		for (char *x = strtok(p, "/"); x; x = strtok(NULL, "/")) {
@@ -239,9 +230,9 @@ setup() {
 
 		// Path <-> file index mapping
 #if LIBTORRENT_VERSION_NUM < 10100
-		files["/" + ti.file_at(i).path] = i;
+		files["/" + ti->file_at(i).path] = i;
 #else
-		files["/" + ti.files().file_path(i)] = i;
+		files["/" + ti->files().file_path(i)] = i;
 #endif
 	}
 }
@@ -286,7 +277,7 @@ handle_torrent_added_alert(libtorrent::torrent_added_alert *a, Log *log) {
 
 	handle = a->handle;
 
-	if (a->handle.status(0).has_metadata)
+	if (a->handle.status().has_metadata)
 		setup();
 
 	pthread_mutex_unlock(&lock);
@@ -425,18 +416,25 @@ btfs_getattr(const char *path, struct stat *stbuf) {
 	if (strcmp(path, "/") == 0 || is_dir(path)) {
 		stbuf->st_mode = S_IFDIR | 0755;
 	} else {
-#if LIBTORRENT_VERSION_NUM < 10000
-		libtorrent::torrent_info ti = handle.get_torrent_info();
-#else
-		libtorrent::torrent_info ti = *handle.torrent_file();
-#endif
+		auto ti = handle.torrent_file();
 
 #if LIBTORRENT_VERSION_NUM < 10100
-		int64_t file_size = ti.file_at(files[path]).size;
+		int64_t file_size = ti->file_at(files[path]).size;
 #else
-		int64_t file_size = ti.files().file_size(files[path]);
+		int64_t file_size = ti->files().file_size(files[path]);
 #endif
 
+#if LIBTORRENT_VERSION_NUM < 10200
+		std::vector<boost::int64_t> progress;
+#else
+		std::vector<std::int64_t> progress;
+#endif
+
+		// Get number of bytes downloaded of each file
+		handle.file_progress(progress,
+			libtorrent::torrent_handle::piece_granularity);
+
+		stbuf->st_blocks = progress[(size_t) files[path]] / 512;
 		stbuf->st_mode = S_IFREG | 0444;
 		stbuf->st_size = file_size;
 	}
@@ -516,6 +514,26 @@ btfs_read(const char *path, char *buf, size_t size, off_t offset,
 	return s;
 }
 
+static int
+btfs_statfs(const char *path, struct statvfs *stbuf) {
+	libtorrent::torrent_status st = handle.status();
+
+	if (!st.has_metadata)
+		return -ENOENT;
+
+	auto ti = handle.torrent_file();
+
+	stbuf->f_bsize = 4096;
+	stbuf->f_frsize = 512;
+	stbuf->f_blocks = (fsblkcnt_t) (ti->total_size() / 512);
+	stbuf->f_bfree = (fsblkcnt_t) ((ti->total_size() - st.total_done) / 512);
+	stbuf->f_bavail = (fsblkcnt_t) ((ti->total_size() - st.total_done) / 512);
+	stbuf->f_files = (fsfilcnt_t) (files.size() + dirs.size());
+	stbuf->f_ffree = 0;
+
+	return 0;
+}
+
 static void *
 btfs_init(struct fuse_conn_info *conn) {
 	pthread_mutex_lock(&lock);
@@ -529,7 +547,11 @@ btfs_init(struct fuse_conn_info *conn) {
 		libtorrent::session::add_default_plugins |
 		libtorrent::session::start_default_features;
 
+#if LIBTORRENT_VERSION_NUM < 10200
 	int alerts =
+#else
+	libtorrent::alert_category_t alerts =
+#endif
 		libtorrent::alert::tracker_notification |
 		libtorrent::alert::stats_notification |
 		libtorrent::alert::storage_notification |
@@ -584,6 +606,13 @@ btfs_init(struct fuse_conn_info *conn) {
 		STRINGIFY(LIBTORRENT_VERSION_MINOR)
 		"00";
 
+#if LIBTORRENT_VERSION_NUM >= 10101
+	pack.set_str(pack.dht_bootstrap_nodes,
+		"router.bittorrent.com:6881,"
+		"router.utorrent.com:6881,"
+		"dht.transmissionbt.com:6881");
+#endif
+
 	pack.set_str(pack.listen_interfaces, interfaces.str());
 	pack.set_bool(pack.strict_end_game_mode, false);
 	pack.set_bool(pack.announce_to_all_trackers, true);
@@ -594,9 +623,12 @@ btfs_init(struct fuse_conn_info *conn) {
 
 	session = new libtorrent::session(pack, flags);
 
+#if LIBTORRENT_VERSION_NUM < 10101
 	session->add_dht_router(std::make_pair("router.bittorrent.com", 6881));
 	session->add_dht_router(std::make_pair("router.utorrent.com", 6881));
 	session->add_dht_router(std::make_pair("dht.transmissionbt.com", 6881));
+#endif
+
 	session->add_torrent(*p);
 #endif
 
@@ -710,10 +742,14 @@ populate_metadata(libtorrent::add_torrent_params& p, const char *arg) {
 #if LIBTORRENT_VERSION_NUM < 10100
 		p.ti = new libtorrent::torrent_info((const char *) output.buf,
 			(int) output.size, ec);
-#else
+#elif LIBTORRENT_VERSION_NUM < 10200
 		p.ti = boost::make_shared<libtorrent::torrent_info>(
 			(const char *) output.buf, (int) output.size,
 			boost::ref(ec));
+#else
+		p.ti = std::make_shared<libtorrent::torrent_info>(
+			(const char *) output.buf, (int) output.size,
+			std::ref(ec));
 #endif
 
 		if (ec)
@@ -740,9 +776,12 @@ populate_metadata(libtorrent::add_torrent_params& p, const char *arg) {
 
 #if LIBTORRENT_VERSION_NUM < 10100
 		p.ti = new libtorrent::torrent_info(r, ec);
-#else
+#elif LIBTORRENT_VERSION_NUM < 10200
 		p.ti = boost::make_shared<libtorrent::torrent_info>(r,
 			boost::ref(ec));
+#else
+		p.ti = std::make_shared<libtorrent::torrent_info>(r,
+			std::ref(ec));
 #endif
 
 		free(r);
@@ -765,6 +804,7 @@ static const struct fuse_opt btfs_opts[] = {
 	BTFS_OPT("--version",                    version,              1),
 	BTFS_OPT("-h",                           help,                 1),
 	BTFS_OPT("--help",                       help,                 1),
+	BTFS_OPT("--help-fuse",                  help_fuse,            1),
 	BTFS_OPT("-b",                           browse_only,          1),
 	BTFS_OPT("--browse-only",                browse_only,          1),
 	BTFS_OPT("-k",                           keep,                 1),
@@ -801,13 +841,13 @@ print_help() {
 	printf("btfs options:\n");
 	printf("    --version -v           show version information\n");
 	printf("    --help -h              show this message\n");
+	printf("    --help-fuse            print all fuse options\n");
 	printf("    --browse-only -b       download metadata only\n");
 	printf("    --keep -k              keep files after unmount\n");
 	printf("    --min-port=N           start of listen port range\n");
 	printf("    --max-port=N           end of listen port range\n");
 	printf("    --max-download-rate=N  max download rate (in kB/s)\n");
 	printf("    --max-upload-rate=N    max upload rate (in kB/s)\n");
-	printf("\n");
 }
 
 int
@@ -819,6 +859,7 @@ main(int argc, char *argv[]) {
 	btfs_ops.readdir = btfs_readdir;
 	btfs_ops.open = btfs_open;
 	btfs_ops.read = btfs_read;
+	btfs_ops.statfs = btfs_statfs;
 	btfs_ops.init = btfs_init;
 	btfs_ops.destroy = btfs_destroy;
 
@@ -841,12 +882,17 @@ main(int argc, char *argv[]) {
 		return 0;
 	}
 
-	if (params.help) {
+	if (params.help || params.help_fuse) {
+		// Print info about btfs' command line options
 		print_help();
 
-		// Let FUSE print more help
-		fuse_opt_add_arg(&args, "-ho");
-		fuse_main(args.argc, args.argv, &btfs_ops, NULL);
+		if (params.help_fuse) {
+			printf("\n");
+
+			// Let FUSE print more help
+			fuse_opt_add_arg(&args, "-ho");
+			fuse_main(args.argc, args.argv, &btfs_ops, NULL);
+		}
 
 		return 0;
 	}
